@@ -1,5 +1,6 @@
 require 'pry/code/loc'
 require 'pry/code/code_range'
+require 'pry/code/code_file'
 
 class Pry
   class << self
@@ -29,26 +30,6 @@ class Pry
   # arbitrary chaining of formatting methods without mutating the original
   # object.
   class Code
-
-    # List of all supported languages.
-    # @return [Hash]
-    EXTENSIONS = {
-      %w(.py)        => :python,
-      %w(.js)        => :javascript,
-      %w(.css)       => :css,
-      %w(.xml)       => :xml,
-      %w(.php)       => :php,
-      %w(.html)      => :html,
-      %w(.diff)      => :diff,
-      %w(.java)      => :java,
-      %w(.json)      => :json,
-      %w(.c .h)      => :c,
-      %w(.rhtml)     => :rhtml,
-      %w(.yaml .yml) => :yaml,
-      %w(.cpp .hpp .cc .h cxx) => :cpp,
-      %w(.rb .ru .irbrc .gemspec .pryrc) => :ruby,
-    }
-
     class << self
       include MethodSource::CodeHelpers
 
@@ -58,13 +39,9 @@ class Pry
       # @param [String] filename The name of a file, or "(pry)".
       # @param [Symbol] code_type The type of code the file contains.
       # @return [Code]
-      def from_file(filename, code_type = type_from_filename(filename))
-        code = if filename == Pry.eval_path
-                 Pry.line_buffer.drop(1)
-               else
-                 File.read(abs_path(filename))
-               end
-        new(code, 1, code_type)
+      def from_file(filename, code_type = nil)
+        code_file = CodeFile.new(filename, code_type)
+        new(code_file.code, 1, code_file.code_type)
       end
 
       # Instantiate a `Code` object containing code extracted from a
@@ -94,35 +71,6 @@ class Pry
         start_line ||= candidate.line
         new(candidate.source, start_line, :ruby)
       end
-
-      protected
-
-      # Guess the CodeRay type of a file from its extension, or nil if
-      # unknown.
-      #
-      # @param [String] filename
-      # @param [Symbol] default (:ruby) the file type to assume if none could be
-      #   detected.
-      # @return [Symbol, nil]
-      def type_from_filename(filename, default = :ruby)
-        _, type = Pry::Code::EXTENSIONS.find do |k, _|
-          k.any? { |ext| ext == File.extname(filename) }
-        end
-
-        type || default
-      end
-
-      # @param [String] filename
-      # @raise [MethodSource::SourceNotFoundError] if the +filename+ is not
-      #   readable for some reason.
-      # @return [String] absolute path for the given +filename+.
-      def abs_path(filename)
-        abs_path = [File.expand_path(filename, Dir.pwd),
-                    File.expand_path(filename, Pry::INITIAL_PWD)
-                   ].detect { |abs_path| File.readable?(abs_path) }
-        abs_path or raise MethodSource::SourceNotFoundError,
-                          "Cannot open #{filename.inspect} for reading."
-      end
     end
 
     # @return [Symbol] The type of code stored in this wrapper.
@@ -197,7 +145,7 @@ class Pry
         if start_line >= 0
           @lines.index { |loc| loc.lineno >= start_line } || @lines.length
         else
-          @lines.length + start_line
+          [@lines.length + start_line, 0].max
         end
 
       alter do
@@ -305,14 +253,27 @@ class Pry
     # @return [String] a formatted representation (based on the configuration of
     #   the object).
     def to_s
-      @lines.map { |loc|
+      print_to_output("", false)
+    end
+
+    # @return [String] a (possibly highlighted) copy of the source code.
+    def highlighted
+      print_to_output("", true)
+    end
+
+    # Writes a formatted representation (based on the configuration of the
+    # object) to the given output, which must respond to `#<<`.
+    def print_to_output(output, color=false)
+      @lines.each do |loc|
         loc = loc.dup
-        loc.colorize(@code_type)              if Pry.color
-        loc.add_line_number(max_lineno_width) if @with_line_numbers
+        loc.colorize(@code_type)              if color
+        loc.add_line_number(max_lineno_width, color) if @with_line_numbers
         loc.add_marker(@marker_lineno)        if @with_marker
         loc.indent(@indentation_num)          if @with_indentation
-        loc.line
-      }.join("\n") + "\n"
+        output << loc.line
+        output << "\n"
+      end
+      output
     end
 
     # Get the comment that describes the expression on the given line number.
@@ -344,7 +305,7 @@ class Pry
     #
     # @return [String]
     def raw
-      @lines.map(&:line).join("\n") + "\n"
+      @lines.map(&:line).join("\n") << "\n"
     end
 
     # Return the number of lines stored.
