@@ -18,11 +18,11 @@ class Pry
     # @param [Hash] hash The hash to convert to `Pry::Hooks`.
     # @return [Pry::Hooks] The resulting `Pry::Hooks` instance.
     def self.from_hash(hash)
+      return hash if hash.instance_of?(self)
       instance = new
       hash.each do |k, v|
         instance.add_hook(k, nil, v)
       end
-
       instance
     end
 
@@ -49,22 +49,6 @@ class Pry
       @errors ||= []
     end
 
-    # FIXME:
-    # This is a hack to alert people of the new API.
-    def [](event_name)
-      warn "`Pry.hooks[]` is deprecated! Please use the new `Pry::Hooks` API! http://rubydoc.info/github/pry/pry/master/Pry/Hooks"
-
-      get_hook(event_name, nil)
-    end
-
-    # FIXME:
-    # This is a hack to alert people of the new API.
-    def []=(event_name, callable)
-      warn "`Pry.hooks[]=` is deprecated! Please use the new `Pry::Hooks` API! http://rubydoc.info/github/pry/pry/master/Pry/Hooks"
-
-      add_hook(event_name, nil, callable)
-    end
-
     # Destructively merge the contents of two `Pry:Hooks` instances.
     # @param [Pry::Hooks] other The `Pry::Hooks` instance to merge
     # @return [Pry:Hooks] Returns the receiver.
@@ -146,18 +130,14 @@ class Pry
     def exec_hook(event_name, *args, &block)
       @hooks[event_name] ||= []
 
-      # silence warnings to get rid of 1.8's "warning: multiple values
-      # for a block parameter" warnings
-      Pry::Helpers::BaseHelpers.silence_warnings do
-        @hooks[event_name].map do |hook_name, callable|
-          begin
-            callable.call(*args, &block)
-          rescue RescuableException => e
-            errors << e
-            e
-          end
-        end.last
-      end
+      @hooks[event_name].map do |hook_name, callable|
+        begin
+          callable.call(*args, &block)
+        rescue RescuableException => e
+          errors << e
+          e
+        end
+      end.last
     end
 
     # Return the number of hook functions registered for the `event_name` event.
