@@ -1,4 +1,4 @@
-require 'helper'
+require_relative 'helper'
 
 describe Pry::CommandSet do
   before do
@@ -8,10 +8,35 @@ describe Pry::CommandSet do
 
     @ctx = {
       :target => binding,
-      :command_set => @set
+      :command_set => @set,
+      :pry_instance => Pry.new(output: StringIO.new)
     }
   end
 
+  describe "[]=" do
+    it "removes a command from the command set" do
+      @set["help"].should_not == nil
+      @set["help"] = nil
+      @set["help"].should == nil
+      expect { @set.run_command(TOPLEVEL_BINDING, "help") }.to raise_error Pry::NoCommandError
+    end
+
+    it "replaces a command" do
+      old_help = @set["help"]
+      @set["help"] = @set["pry-version"]
+      @set["help"].should_not == old_help
+    end
+
+    it "rebinds the command with key" do
+      @set["help-1"] = @set["help"]
+      @set["help-1"].match.should == "help-1"
+    end
+
+    it "raises a TypeError when command is not a subclass of Pry::Command" do
+      expect { @set["help"] = "hello" }.to raise_error TypeError
+    end
+  end
+
   it 'should call the block used for the command when it is called' do
     run = false
     @set.command 'foo' do
@@ -42,27 +67,21 @@ describe Pry::CommandSet do
 
   it 'should raise an error when calling an undefined command' do
     @set.command('foo') {}
-    lambda {
-      @set.run_command @ctx, 'bar'
-    }.should.raise(Pry::NoCommandError)
+    expect { @set.run_command @ctx, 'bar' }.to raise_error Pry::NoCommandError
   end
 
   it 'should be able to remove its own commands' do
     @set.command('foo') {}
     @set.delete 'foo'
 
-    lambda {
-      @set.run_command @ctx, 'foo'
-    }.should.raise(Pry::NoCommandError)
+    expect { @set.run_command @ctx, 'foo' }.to raise_error Pry::NoCommandError
   end
 
   it 'should be able to remove its own commands, by listing name' do
     @set.command(/^foo1/, 'desc', :listing => 'foo') {}
     @set.delete 'foo'
 
-    lambda {
-      @set.run_command @ctx, /^foo1/
-    }.should.raise(Pry::NoCommandError)
+    expect { @set.run_command @ctx, /^foo1/ }.to raise_error Pry::NoCommandError
   end
 
   it 'should be able to import some commands from other sets' do
@@ -78,9 +97,7 @@ describe Pry::CommandSet do
     @set.run_command @ctx, 'foo'
     run.should == true
 
-    lambda {
-      @set.run_command @ctx, 'bar'
-    }.should.raise(Pry::NoCommandError)
+    expect { @set.run_command @ctx, 'bar' }.to raise_error Pry::NoCommandError
   end
 
   it 'should return command set after import' do
@@ -143,7 +160,7 @@ describe Pry::CommandSet do
 
   it 'should set the descriptions of commands' do
     @set.command('foo', 'some stuff') {}
-    @set.commands['foo'].description.should == 'some stuff'
+    @set['foo'].description.should == 'some stuff'
   end
 
   describe "aliases" do
@@ -152,24 +169,42 @@ describe Pry::CommandSet do
       @set.command('foo', 'stuff') { run = true }
 
       @set.alias_command 'bar', 'foo'
-      @set.commands['bar'].match.should == 'bar'
-      @set.commands['bar'].description.should == 'Alias for `foo`'
+      @set['bar'].match.should == 'bar'
+      @set['bar'].description.should == 'Alias for `foo`'
 
       @set.run_command @ctx, 'bar'
       run.should == true
     end
 
+    it "should be able to alias command with command_prefix" do
+      run = false
+
+      begin
+        @set.command('owl', 'stuff') { run = true }
+        @set.alias_command 'owlet', 'owl'
+
+        Pry.config.command_prefix = '%'
+        @set['%owlet'].match.should == 'owlet'
+        @set['%owlet'].description.should == 'Alias for `owl`'
+
+        @set.run_command @ctx, 'owlet'
+        run.should == true
+      ensure
+        Pry.config.command_prefix = ''
+      end
+    end
+
     it 'should inherit options from original command' do
       run = false
       @set.command('foo', 'stuff', :shellwords => true, :interpolate => false) { run = true }
 
       @set.alias_command 'bar', 'foo'
-      @set.commands['bar'].options[:shellwords].should == @set.commands['foo'].options[:shellwords]
-      @set.commands['bar'].options[:interpolate].should == @set.commands['foo'].options[:interpolate]
+      @set['bar'].options[:shellwords].should == @set['foo'].options[:shellwords]
+      @set['bar'].options[:interpolate].should == @set['foo'].options[:interpolate]
 
       # however some options should not be inherited
-      @set.commands['bar'].options[:listing].should.not ==  @set.commands['foo'].options[:listing]
-      @set.commands['bar'].options[:listing].should == "bar"
+      @set['bar'].options[:listing].should_not ==  @set['foo'].options[:listing]
+      @set['bar'].options[:listing].should == "bar"
     end
 
     it 'should be able to specify alias\'s description when aliasing' do
@@ -177,8 +212,8 @@ describe Pry::CommandSet do
       @set.command('foo', 'stuff') { run = true }
 
       @set.alias_command 'bar', 'foo', :desc => "tobina"
-      @set.commands['bar'].match.should == 'bar'
-      @set.commands['bar'].description.should == "tobina"
+      @set['bar'].match.should == 'bar'
+      @set['bar'].description.should == "tobina"
 
       @set.run_command @ctx, 'bar'
       run.should == true
@@ -189,8 +224,8 @@ describe Pry::CommandSet do
       @set.command(/^foo1/, 'stuff', :listing => 'foo') { run = true }
 
       @set.alias_command 'bar', 'foo1'
-      @set.commands['bar'].match.should == 'bar'
-      @set.commands['bar'].description.should == 'Alias for `foo1`'
+      @set['bar'].match.should == 'bar'
+      @set['bar'].description.should == 'Alias for `foo1`'
 
       @set.run_command @ctx, 'bar'
       run.should == true
@@ -201,7 +236,7 @@ describe Pry::CommandSet do
       @set.command(/^foo1/, 'stuff', :listing => 'foo') { run = true }
 
       @set.alias_command /^b.r/, 'foo1', :listing => "bar"
-      @set.commands[/^b.r/].options[:listing].should == "bar"
+      @set.to_hash[/^b.r/].options[:listing].should == "bar"
     end
 
     it "should set description to default if description parameter is nil" do
@@ -209,7 +244,7 @@ describe Pry::CommandSet do
       @set.command(/^foo1/, 'stuff', :listing => 'foo') { run = true }
 
       @set.alias_command "bar", 'foo1'
-      @set.commands["bar"].description.should == "Alias for `foo1`"
+      @set["bar"].description.should == "Alias for `foo1`"
     end
   end
 
@@ -217,7 +252,7 @@ describe Pry::CommandSet do
     @set.command('foo', 'bar') {}
     @set.desc 'foo', 'baz'
 
-    @set.commands['foo'].description.should == 'baz'
+    @set['foo'].description.should == 'baz'
   end
 
   it 'should get the descriptions of commands' do
@@ -246,22 +281,19 @@ describe Pry::CommandSet do
   end
 
   it 'should be able to have its own helpers' do
-    @set.command('foo') do
-      should.respond_to :my_helper
-    end
-
-    @set.helpers do
-      def my_helper; end
-    end
+    @set.command('foo') { my_helper }
+    @set.helpers { def my_helper; end }
 
     @set.run_command(@ctx, 'foo')
-    Pry::Command.subclass('foo', '', {}, Module.new).new({:target => binding}).should.not.respond_to :my_helper
+    Pry::Command.subclass('foo', '', {}, Module.new)
+                .new({:target => binding})
+                .should_not(respond_to :my_helper)
   end
 
   it 'should not recreate a new helper module when helpers is called' do
     @set.command('foo') do
-      should.respond_to :my_helper
-      should.respond_to :my_other_helper
+      my_helper
+      my_other_helper
     end
 
     @set.helpers do
@@ -283,7 +315,7 @@ describe Pry::CommandSet do
     end
 
     @set.import imported_set
-    @set.command('foo') { should.respond_to :imported_helper_method }
+    @set.command('foo') { imported_helper_method }
     @set.run_command(@ctx, 'foo')
   end
 
@@ -297,27 +329,25 @@ describe Pry::CommandSet do
     end
 
     @set.import_from imported_set, 'bar'
-    @set.command('foo') { should.respond_to :imported_helper_method }
+    @set.command('foo') { imported_helper_method }
     @set.run_command(@ctx, 'foo')
   end
 
   it 'should provide a :listing for a command that defaults to its name' do
     @set.command 'foo', "" do;end
-    @set.commands['foo'].options[:listing].should == 'foo'
+    @set['foo'].options[:listing].should == 'foo'
   end
 
   it 'should provide a :listing for a command that differs from its name' do
     @set.command 'foo', "", :listing => 'bar' do;end
-    @set.commands['foo'].options[:listing].should == 'bar'
+    @set['foo'].options[:listing].should == 'bar'
   end
 
   it "should provide a 'help' command" do
     @ctx[:command_set] = @set
     @ctx[:output] = StringIO.new
 
-    lambda {
-      @set.run_command(@ctx, 'help')
-    }.should.not.raise
+    expect { @set.run_command(@ctx, 'help') }.to_not raise_error
   end
 
 
@@ -339,13 +369,13 @@ describe Pry::CommandSet do
     end
 
     it 'should raise exception trying to rename non-existent command' do
-      lambda { @set.rename_command('bar', 'foo') }.should.raise ArgumentError
+      expect { @set.rename_command('bar', 'foo') }.to raise_error ArgumentError
     end
 
     it 'should make old command name inaccessible' do
       @set.command('foo') { }
       @set.rename_command('bar', 'foo')
-      lambda { @set.run_command(@ctx, 'foo') }.should.raise Pry::NoCommandError
+      expect { @set.run_command(@ctx, 'foo') }.to raise_error Pry::NoCommandError
     end
 
     it 'should be able to pass in options when renaming command' do
@@ -353,9 +383,9 @@ describe Pry::CommandSet do
       listing = "bing"
       @set.command('foo') { }
       @set.rename_command('bar', 'foo', :description => desc, :listing => listing, :keep_retval => true)
-      @set.commands['bar'].description.should           == desc
-      @set.commands['bar'].options[:listing].should     == listing
-      @set.commands['bar'].options[:keep_retval].should == true
+      @set['bar'].description.should           == desc
+      @set['bar'].options[:listing].should     == listing
+      @set['bar'].options[:keep_retval].should == true
     end
   end
 
@@ -505,17 +535,23 @@ describe Pry::CommandSet do
     end
 
     it 'should not find commands without command_prefix' do
-      Pry.config.command_prefix = '%'
-      cmd = @set.command('detritus'){ }
-      @set.find_command('detritus').should == nil
-      Pry.config.command_prefix = ''
+      begin
+        Pry.config.command_prefix = '%'
+        cmd = @set.command('detritus'){ }
+        @set.find_command('detritus').should == nil
+      ensure
+        Pry.config.command_prefix = ''
+      end
     end
 
     it "should find commands that don't use the prefix" do
-      Pry.config.command_prefix = '%'
-      cmd = @set.command('colon', 'Sergeant Fred', :use_prefix => false){ }
-      @set.find_command('colon').should == cmd
-      Pry.config.command_prefix = ''
+      begin
+        Pry.config.command_prefix = '%'
+        cmd = @set.command('colon', 'Sergeant Fred', :use_prefix => false){ }
+        @set.find_command('colon').should == cmd
+      ensure
+        Pry.config.command_prefix = ''
+      end
     end
 
     it "should find the command that has the longest match" do
@@ -543,9 +579,7 @@ describe Pry::CommandSet do
 
     it 'should not cause argument interpolation' do
       cmd = @set.command('hello')
-      lambda {
-        @set.valid_command?('hello #{raise "futz"}')
-      }.should.not.raise
+      expect { @set.valid_command?('hello #{raise "futz"}') }.to_not raise_error
     end
   end
 
