@@ -1,4 +1,4 @@
-require 'helper'
+require_relative 'helper'
 
 describe "Pry::Command" do
 
@@ -22,9 +22,7 @@ describe "Pry::Command" do
         #
       end
 
-      lambda {
-        mock_command(cmd, %w())
-      }.should.raise(Pry::CommandError)
+      expect { mock_command(cmd, %w()) }.to raise_error Pry::CommandError
     end
 
     it 'should return VOID without keep_retval' do
@@ -94,7 +92,7 @@ describe "Pry::Command" do
         #
       end
 
-      mock_command(@set.commands['help'], %w(oolon-colluphid), :command_set => @set).output.should =~ /Raving Atheist/
+      mock_command(@set['help'], %w(oolon-colluphid), :command_set => @set).output.should =~ /Raving Atheist/
     end
 
     it 'should use slop to generate the help for classy commands' do
@@ -104,7 +102,7 @@ describe "Pry::Command" do
         end
       end
 
-      mock_command(@set.commands['help'], %w(eddie), :command_set => @set).output.should =~ /Over-cheerful/
+      mock_command(@set['help'], %w(eddie), :command_set => @set).output.should =~ /Over-cheerful/
     end
 
     it 'should provide --help for classy commands' do
@@ -144,7 +142,7 @@ describe "Pry::Command" do
       :output => StringIO.new,
       :eval_string => "eval-string",
       :command_set => @set,
-      :pry_instance => Object.new
+      :pry_instance => Pry.new
     }
 
     it 'should capture lots of stuff from the hash passed to new before setup' do
@@ -198,9 +196,7 @@ describe "Pry::Command" do
         end
       end
 
-      lambda {
-        mock_command(cmd)
-      }.should.raise(Pry::CommandError)
+      expect { mock_command(cmd) }.to raise_error Pry::CommandError
     end
 
     it 'should work if neither options, nor setup is overridden' do
@@ -402,10 +398,7 @@ describe "Pry::Command" do
 
     it "should raise an error if the line doesn't match the command" do
       cmd = @set.command 'grunthos', 'the flatulent'
-
-      lambda {
-        cmd.new.process_line %(grumpos)
-      }.should.raise(Pry::CommandError)
+      expect { cmd.new.process_line %(grumpos) }.to raise_error Pry::CommandError
     end
    end
 
@@ -619,9 +612,7 @@ describe "Pry::Command" do
           end
           @out = StringIO.new
 
-          proc {
-            @t.eval 'walking-spanish | { :jesus }'
-          }.should.raise(NoMethodError)
+          expect { @t.eval 'walking-spanish | { :jesus }' }.to raise_error(NoMethodError)
         end
 
         it "should expose block in command_block method" do
@@ -653,21 +644,19 @@ describe "Pry::Command" do
         end
       end
 
-      Pry.commands.add_command MyTestCommand
+      Pry.config.commands.add_command MyTestCommand
     end
 
     after do
-      Pry.commands.delete 'my-test'
+      Pry.config.commands.delete 'my-test'
     end
 
     it "allows creation of custom subclasses of Pry::Command" do
       pry_eval('my---test').should =~ /my-testmy-test/
     end
 
-    if !mri18_and_no_real_source_location?
-      it "shows the source of the process method" do
-        pry_eval('show-source my-test').should =~ /output.puts command_name/
-      end
+    it "shows the source of the process method" do
+      pry_eval('show-source my-test').should =~ /output.puts command_name/
     end
 
     describe "command options hash" do
@@ -691,11 +680,11 @@ describe "Pry::Command" do
             match 'happy-new-year'
             description 'Happy New Year 2013'
           end
-          Pry.commands.add_command HappyNewYear
+          Pry.config.commands.add_command HappyNewYear
 
           HappyNewYear.options[:listing].should == 'happy-new-year'
 
-          Pry.commands.delete 'happy-new-year'
+          Pry.config.commands.delete 'happy-new-year'
         end
 
         it "can be set explicitly" do
@@ -704,23 +693,23 @@ describe "Pry::Command" do
             description 'Merry Christmas!'
             command_options :listing => 'happy-holidays'
           end
-          Pry.commands.add_command MerryChristmas
+          Pry.config.commands.add_command MerryChristmas
 
           MerryChristmas.options[:listing].should == 'happy-holidays'
 
-          Pry.commands.delete 'merry-christmas'
+          Pry.config.commands.delete 'merry-christmas'
         end
 
         it "equals to :match option's inspect, if :match is Regexp" do
           class CoolWinter < Pry::ClassCommand
-            match /.*winter/
+            match(/.*winter/)
             description 'Is winter cool or cool?'
           end
-          Pry.commands.add_command CoolWinter
+          Pry.config.commands.add_command CoolWinter
 
           CoolWinter.options[:listing].should == '/.*winter/'
 
-          Pry.commands.delete /.*winter/
+          Pry.config.commands.delete /.*winter/
         end
       end
     end
@@ -805,17 +794,17 @@ describe "Pry::Command" do
     end
 
     it 'should be correct for default commands' do
-      @set.commands["help"].group.should == "Help"
+      @set["help"].group.should == "Help"
     end
 
     it 'should not change once it is initialized' do
-      @set.commands["magic"].group("-==CD COMMAND==-")
-      @set.commands["magic"].group.should == "Not for a public use"
+      @set["magic"].group("-==CD COMMAND==-")
+      @set["magic"].group.should == "Not for a public use"
     end
 
     it 'should not disappear after the call without parameters' do
-      @set.commands["magic"].group
-      @set.commands["magic"].group.should == "Not for a public use"
+      @set["magic"].group
+      @set["magic"].group.should == "Not for a public use"
     end
   end
 end
