@@ -1,10 +1,8 @@
-require 'helper'
+require_relative '../helper'
 
 describe "exit-program" do
   it 'should raise SystemExit' do
-    proc {
-      pry_eval('exit-program')
-    }.should.raise SystemExit
+    expect { pry_eval('exit-program') }.to raise_error SystemExit
   end
 
   it 'should exit the program with the provided value' do
