@@ -1,10 +1,10 @@
-require 'helper'
+require_relative '../helper'
 
 describe "ls" do
   describe "below ceiling" do
     it "should stop before Object by default" do
-      pry_eval("cd Class.new{ def goo; end }.new", "ls").should.not =~ /Object/
-      pry_eval("cd Class.new{ def goo; end }", "ls -M").should.not =~ /Object/
+      pry_eval("cd Class.new{ def goo; end }.new", "ls").should_not =~ /Object/
+      pry_eval("cd Class.new{ def goo; end }", "ls -M").should_not =~ /Object/
     end
 
     it "should include object if -v is given" do
@@ -23,8 +23,8 @@ describe "ls" do
     end
 
     it "should not include super-classes when -q is given" do
-      pry_eval("cd Class.new(Class.new{ def goo; end }).new", "ls -q").should.not =~ /goo/
-      pry_eval("cd Class.new(Class.new{ def goo; end })", "ls -M -q").should.not =~ /goo/
+      pry_eval("cd Class.new(Class.new{ def goo; end }).new", "ls -q").should_not =~ /goo/
+      pry_eval("cd Class.new(Class.new{ def goo; end })", "ls -M -q").should_not =~ /goo/
     end
   end
 
@@ -34,6 +34,25 @@ describe "ls" do
     end
   end
 
+  describe "BasicObject" do
+    it "should work on BasicObject" do
+      pry_eval("ls BasicObject.new").should =~ /BasicObject#methods:.*__send__/m
+    end
+
+    it "should work on subclasses of BasicObject" do
+      pry_eval(
+        "class LessBasic < BasicObject; def jaroussky; 5; end; end",
+        "ls LessBasic.new"
+      ).should =~ /LessBasic#methods:.*jaroussky/m
+    end
+  end
+
+  describe "immediates" do
+    it "should work on Fixnum" do
+      pry_eval("ls 5").should =~ /Fixnum#methods:.*modulo/m
+    end
+  end
+
   describe "methods" do
     it "should show public methods by default" do
       output = pry_eval("ls Class.new{ def goo; end; public :goo }.new")
@@ -41,8 +60,8 @@ describe "ls" do
     end
 
     it "should not show protected/private by default" do
-      pry_eval("ls -M Class.new{ def goo; end; private :goo }").should.not =~ /goo/
-      pry_eval("ls Class.new{ def goo; end; protected :goo }.new").should.not =~ /goo/
+      pry_eval("ls -M Class.new{ def goo; end; private :goo }").should_not =~ /goo/
+      pry_eval("ls Class.new{ def goo; end; protected :goo }.new").should_not =~ /goo/
     end
 
     it "should show public methods with -p" do
@@ -59,17 +78,47 @@ describe "ls" do
       # This doesn't actually touch the network, promise!
       pry_eval("ls Net::HTTP::Get.new('localhost')").should =~ /Net::HTTPGenericRequest#methods/
     end
+
+    it "should work for objects which instance_variables returns array of symbol but there is no Symbol#downcase" do
+      test_case = "class Object; alias :fg :instance_variables; def instance_variables; fg.map(&:to_sym); end end;"
+      normalize = "class Object; def instance_variables; fg; end end;"
+
+      test = lambda do
+        begin
+          pry_eval(test_case, "class GeFromulate2; @flurb=1.3; end", "cd GeFromulate2", "ls")
+          pry_eval(normalize)
+        rescue
+          pry_eval(normalize)
+          raise
+        end
+      end
+
+      expect(test).to_not raise_error
+    end
+
+    it "should show error message when instance is given with -M option" do
+      expect { pry_eval("ls -M String.new") }.to raise_error(Pry::CommandError, /-M only makes sense with a Module or a Class/)
+    end
+
+
+    # see: https://travis-ci.org/pry/pry/jobs/5071918
+    unless Pry::Helpers::BaseHelpers.rbx?
+      it "should handle classes that (pathologically) define .ancestors" do
+        output = pry_eval("ls Class.new{ def self.ancestors; end; def hihi; end }")
+        output.should =~ /hihi/
+      end
+    end
   end
 
   describe 'with -l' do
-    it 'should find locals and sort by descending size' do
-      result = pry_eval("aa = 'asdf'; bb = 'xyz'", 'ls -l')
-      result.should.not =~ /=>/
-      result.should.not =~ /0x\d{5}/
+    focus 'should find locals and sort by descending size' do
+      result = pry_eval(Object.new, "aa = 'asdf'; bb = 'xyz'", 'ls -l')
+      result.should_not =~ /=>/
+      result.should_not =~ /0x\d{5}/
       result.should =~ /asdf.*xyz/m
     end
     it 'should not list pry noise' do
-      pry_eval('ls -l').should.not =~ /_(?:dir|file|ex|pry|out|in)_/
+      pry_eval('ls -l').should_not =~ /_(?:dir|file|ex|pry|out|in)_/
     end
   end
 
@@ -95,17 +144,24 @@ describe "ls" do
     end
 
     it "should behave normally when invoked on Module itself" do
-      pry_eval("ls Module").should.not =~ /Pry/
+      pry_eval("ls Module").should_not =~ /Pry/
     end
   end
 
   describe "constants" do
+    it "works on top-level" do
+      toplevel_consts = pry_eval('ls -c')
+      [/RUBY_PLATFORM/, /ARGF/, /STDOUT/].each do |const|
+        toplevel_consts.should =~ const
+      end
+    end
+
     it "should show constants defined on the current module" do
       pry_eval("class TempFoo1; BARGHL = 1; end", "ls TempFoo1").should =~ /BARGHL/
     end
 
     it "should not show constants defined on parent modules by default" do
-      pry_eval("class TempFoo2; LHGRAB = 1; end; class TempFoo3 < TempFoo2; BARGHL = 1; end", "ls TempFoo3").should.not =~ /LHGRAB/
+      pry_eval("class TempFoo2; LHGRAB = 1; end; class TempFoo3 < TempFoo2; BARGHL = 1; end", "ls TempFoo3").should_not =~ /LHGRAB/
     end
 
     it "should show constants defined on ancestors with -v" do
@@ -114,18 +170,26 @@ describe "ls" do
 
     it "should not autoload constants!" do
       autoload :McflurgleTheThird, "/tmp/this-file-d000esnat-exist.rb"
-      lambda{ pry_eval("ls -c") }.should.not.raise
+      expect { pry_eval("ls -c") }.to_not raise_error
+    end
+
+    it "should show constants for an object's class regardless of mixins" do
+      pry_eval(
+        "cd Pry.new",
+        "extend Module.new",
+        "ls -c"
+      ).should match(/Method/)
     end
   end
 
   describe "grep" do
     it "should reduce the number of outputted things" do
-      pry_eval("ls -c").should =~ /ArgumentError/
-      pry_eval("ls -c --grep Run").should.not =~ /ArgumentError/
+      pry_eval("ls -c Object").should =~ /ArgumentError/
+      pry_eval("ls -c Object --grep Run").should_not =~ /ArgumentError/
     end
 
     it "should still output matching things" do
-      pry_eval("ls -c --grep Run").should =~ /RuntimeError/
+      pry_eval("ls -c Object --grep Run").should =~ /RuntimeError/
     end
   end
 
@@ -169,7 +233,7 @@ describe "ls" do
     describe 'on java objects' do
       it 'should omit java-esque aliases by default' do
         pry_eval('ls java.lang.Thread.current_thread').should =~ /\bthread_group\b/
-        pry_eval('ls java.lang.Thread.current_thread').should.not =~ /\bgetThreadGroup\b/
+        pry_eval('ls java.lang.Thread.current_thread').should_not =~ /\bgetThreadGroup\b/
       end
 
       it 'should include java-esque aliases if requested' do
