@@ -1,4 +1,4 @@
-require 'helper'
+require_relative 'helper'
 
 describe Pry::Helpers::DocumentationHelpers do
   before do
@@ -25,15 +25,15 @@ describe Pry::Helpers::DocumentationHelpers do
 
   describe "process_rdoc" do
     before do
-      Pry.color = true
+      Pry.config.color = true
     end
 
     after do
-      Pry.color = false
+      Pry.config.color = false
     end
 
     it "should syntax highlight indented code" do
-      @helper.process_rdoc("  4 + 4\n").should.not == "  4 + 4\n"
+      @helper.process_rdoc("  4 + 4\n").should_not == "  4 + 4\n"
     end
 
     it "should highlight words surrounded by +s" do
@@ -63,11 +63,6 @@ describe Pry::Helpers::DocumentationHelpers do
     it "should not remove ++" do
       @helper.process_rdoc("--\n  comment in a bubble\n++").should =~ /\+\+/
     end
-
-    it "should do nothing if Pry.color is false" do
-      Pry.color = false
-      @helper.process_rdoc("  4 + 4\n").should == "  4 + 4\n"
-    end
   end
 
-end
\ No newline at end of file
+end
