@@ -1,4 +1,4 @@
-require 'helper'
+require_relative 'helper'
 
 describe "Pry.config.exception_whitelist" do
   before do
@@ -7,13 +7,13 @@ describe "Pry.config.exception_whitelist" do
 
   it 'should rescue all exceptions NOT specified on whitelist' do
     Pry.config.exception_whitelist.include?(NameError).should == false
-    lambda { Pry.start(self, :input => StringIO.new("raise NameError\nexit"), :output => @str_output) }.should.not.raise NameError
+    expect { Pry.start(self, :input => StringIO.new("raise NameError\nexit"), :output => @str_output) }.not_to raise_error
   end
 
   it 'should NOT rescue exceptions specified on whitelist' do
     old_whitelist = Pry.config.exception_whitelist
     Pry.config.exception_whitelist = [NameError]
-    lambda { Pry.start(self, :input => StringIO.new("raise NameError"), :output => @str_output) }.should.raise NameError
+    expect { Pry.start(self, :input => StringIO.new("raise NameError"), :output => @str_output) }.to raise_error NameError
     Pry.config.exception_whitelist = old_whitelist
   end
 end
