@@ -1,13 +1,13 @@
-unless Object.const_defined? 'Pry'
-  $:.unshift File.expand_path '../../lib', __FILE__
-  require 'pry'
-end
-
-require 'mocha/api'
-
+require 'bundler/setup'
 require 'pry/test/helper'
-require 'helpers/bacon'
-require 'helpers/mock_pry'
+Bundler.require :default, :test
+require_relative 'spec_helpers/mock_pry'
+require_relative 'spec_helpers/repl_tester'
+
+if ENV["COVERAGE"]
+  require "simplecov"
+  SimpleCov.start
+end
 
 class Module
   public :remove_const
@@ -18,10 +18,9 @@ end
 # in tests)
 $VERBOSE = nil
 
-Pad = OpenStruct.new
-def Pad.clear
-  @table = {}
-end
+Pad = Class.new do
+  include Pry::Config::Behavior
+end.new(nil)
 
 # to help with tracking down bugs that cause an infinite loop in the test suite
 if ENV["SET_TRACE_FUNC"]
@@ -32,3 +31,8 @@ if ENV["SET_TRACE_FUNC"]
 end
 
 puts "Ruby v#{RUBY_VERSION} (#{defined?(RUBY_ENGINE) ? RUBY_ENGINE : "ruby"}), Pry v#{Pry::VERSION}, method_source v#{MethodSource::VERSION}, CodeRay v#{CodeRay::VERSION}, Slop v#{Slop::VERSION}"
+
+RSpec.configure do |config|
+  config.alias_example_to :should
+  config.include PryTestHelpers
+end
