@@ -1,4 +1,4 @@
-require 'helper'
+require_relative 'helper'
 require 'set'
 
 describe Pry::Method do
@@ -68,6 +68,18 @@ describe Pry::Method do
       meth.name.should == "hello"
     end
 
+    it 'should take care of corner cases like mongo[] e.g Foo::Bar.new[]- issue 998' do
+      klass = Class.new { def []; :hello; end }
+      meth = Pry::Method.from_str("klass.new[]", Pry.binding_for(binding))
+      meth.name.should == "[]"
+    end
+
+    it 'should take care of cases like $ mongo[] - issue 998' do
+      f = Class.new { def []; :hello; end }.new
+      meth = Pry::Method.from_str("f[]", Pry.binding_for(binding))
+      meth.name.should == "[]"
+    end
+
     it 'should look up instance methods using klass.meth#method syntax' do
       klass = Class.new { def self.meth; Class.new; end }
       meth = Pry::Method.from_str("klass.meth#initialize", Pry.binding_for(binding))
@@ -81,7 +93,7 @@ describe Pry::Method do
     end
 
     it 'should not raise an exception if receiver does not exist' do
-      lambda { Pry::Method.from_str("random_klass.meth", Pry.binding_for(binding)) }.should.not.raise
+      expect { Pry::Method.from_str("random_klass.meth", Pry.binding_for(binding)) }.to_not raise_error
     end
   end
 
@@ -132,7 +144,8 @@ describe Pry::Method do
       m.name.should == "gag"
     end
 
-    if defined?(BasicObject) && !Pry::Helpers::BaseHelpers.rbx? # rubinius issue 1921
+    # Temporarily disabled to work around rubinius/rubinius#2871.
+    unless Pry::Helpers::BaseHelpers.rbx?
       it "should find the right method from a BasicObject" do
         a = Class.new(BasicObject) { def gag; ::Kernel.binding; end; def self.line; __LINE__; end }
 
@@ -215,7 +228,7 @@ describe Pry::Method do
 
   describe 'all_from_class' do
     def should_find_method(name)
-      Pry::Method.all_from_class(@class).map(&:name).should.include(name)
+      Pry::Method.all_from_class(@class).map(&:name).should include name
     end
 
     it 'should be able to find public instance methods defined in a class' do
@@ -268,7 +281,7 @@ describe Pry::Method do
   describe 'all_from_obj' do
     describe 'on normal objects' do
       def should_find_method(name)
-        Pry::Method.all_from_obj(@obj).map(&:name).should.include(name)
+        Pry::Method.all_from_obj(@obj).map(&:name).should include name
       end
 
       it "should find methods defined in the object's class" do
@@ -300,7 +313,7 @@ describe Pry::Method do
 
       it "should not find methods defined on the classes singleton class" do
         @obj = Class.new{ class << self; def meth; 1; end; end }.new
-        Pry::Method.all_from_obj(@obj).map(&:name).should.not.include('meth')
+        Pry::Method.all_from_obj(@obj).map(&:name).should_not include 'meth'
       end
 
       it "should work in the face of an overridden send" do
@@ -311,7 +324,7 @@ describe Pry::Method do
 
     describe 'on classes' do
       def should_find_method(name)
-        Pry::Method.all_from_obj(@class).map(&:name).should.include(name)
+        Pry::Method.all_from_obj(@class).map(&:name).should include name
       end
 
       it "should find methods defined in the class' singleton class" do
@@ -331,7 +344,7 @@ describe Pry::Method do
 
       it "should not find methods defined within the class" do
         @class = Class.new{ def meth; 1; end }
-        Pry::Method.all_from_obj(@obj).map(&:name).should.not.include('meth')
+        Pry::Method.all_from_obj(@obj).map(&:name).should_not include 'meth'
       end
 
       it "should find methods defined on Class" do
@@ -376,7 +389,7 @@ describe Pry::Method do
         class Bottom < Lower; extend O; end
       end
 
-      def singleton_class(obj); class << obj; self; end; end
+      def singleton_class2(obj); class << obj; self; end; end
 
       it "should look at a class and then its superclass" do
         Pry::Method.instance_resolution_order(LS::Next).should == [LS::Next] + Pry::Method.instance_resolution_order(LS::Top)
@@ -396,7 +409,7 @@ describe Pry::Method do
 
       it "should include the singleton class of objects" do
         obj = LS::Low.new
-        Pry::Method.resolution_order(obj).should == [singleton_class(obj)] + Pry::Method.instance_resolution_order(LS::Low)
+        Pry::Method.resolution_order(obj).should == [singleton_class2(obj)] + Pry::Method.instance_resolution_order(LS::Low)
       end
 
       it "should not include singleton classes of numbers" do
@@ -404,11 +417,11 @@ describe Pry::Method do
       end
 
       it "should include singleton classes for classes" do
-        Pry::Method.resolution_order(LS::Low).should == [singleton_class(LS::Low)] + Pry::Method.resolution_order(LS::Next)
+        Pry::Method.resolution_order(LS::Low).should == [singleton_class2(LS::Low)] + Pry::Method.resolution_order(LS::Next)
       end
 
       it "should include modules included into singleton classes" do
-        Pry::Method.resolution_order(LS::Lower).should == [singleton_class(LS::Lower), LS::N, LS::M] + Pry::Method.resolution_order(LS::Low)
+        Pry::Method.resolution_order(LS::Lower).should == [singleton_class2(LS::Lower), LS::N, LS::M] + Pry::Method.resolution_order(LS::Low)
       end
 
       it "should include modules at most once" do
@@ -416,13 +429,13 @@ describe Pry::Method do
       end
 
       it "should include modules at the point which they would be reached" do
-        Pry::Method.resolution_order(LS::Bottom).should == [singleton_class(LS::Bottom), LS::O] + (Pry::Method.resolution_order(LS::Lower))
+        Pry::Method.resolution_order(LS::Bottom).should == [singleton_class2(LS::Bottom), LS::O] + (Pry::Method.resolution_order(LS::Lower))
       end
 
       it "should include the Pry::Method.instance_resolution_order of Class after the singleton classes" do
         Pry::Method.resolution_order(LS::Top).should ==
-          [singleton_class(LS::Top), singleton_class(Object), (defined? BasicObject) && singleton_class(BasicObject)].compact +
-          Pry::Method.instance_resolution_order(Class)
+          [singleton_class2(LS::Top), singleton_class2(Object), singleton_class2(BasicObject),
+           *Pry::Method.instance_resolution_order(Class)]
       end
     end
   end
@@ -460,23 +473,35 @@ describe Pry::Method do
 
     it 'should return an empty Array if cannot find aliases' do
       meth = Pry::Method(@class.new.method(:eruct))
-      meth.aliases.should.be.empty
+      meth.aliases.should be_empty
     end
 
     it 'should not include the own name in the list of aliases' do
       meth = Pry::Method(@class.new.method(:eat))
-      meth.aliases.should.not.include "eat"
+      meth.aliases.should_not include "eat"
     end
 
-    unless Pry::Helpers::BaseHelpers.mri_18?
-      # Ruby 1.8 doesn't support this feature.
-      it 'should be able to find aliases for methods implemented in C' do
-        meth = Pry::Method(Hash.new.method(:key?))
-        aliases = Set.new(meth.aliases)
+    it 'should find aliases for top-level methods' do
+      # top-level methods get added as private instance methods on Object
+      class Object
+        private
+        def my_top_level_method ; end
+        alias my_other_top_level_method my_top_level_method
+      end
 
-        aliases.should == Set.new(["include?", "member?", "has_key?"])
+      meth = Pry::Method.new(method(:my_top_level_method))
+      meth.aliases.should include 'my_other_top_level_method'
+
+      class Object
+        remove_method :my_top_level_method
       end
     end
 
+    it 'should be able to find aliases for methods implemented in C' do
+      meth = Pry::Method(Hash.new.method(:key?))
+      aliases = Set.new(meth.aliases)
+
+      aliases.should == Set.new(["include?", "member?", "has_key?"])
+    end
   end
 end
